@@ -12,7 +12,8 @@
 
 #include <QtGui/private/qguiapplication_p.h>
 #include <QtGui/private/qpointingdevice_p.h>
-#include <qpa/qplatformtheme.h>
+#include <QtGui/qpa/qplatformtheme.h>
+#include <QtGui/qpa/qwindowsysteminterface_p.h>
 
 #include <wayland-cursor.h>
 
@@ -22,9 +23,6 @@ namespace QtWaylandClient {
 
 using namespace Qt::StringLiterals;
 
-Q_LOGGING_CATEGORY(lcQpaInputDevices, "qt.qpa.input.devices")
-Q_DECLARE_LOGGING_CATEGORY(lcQpaWaylandInput)
-
 #if QT_CONFIG(cursor)
 int QWaylandTabletToolV2::idealCursorScale() const
 {
@@ -187,12 +185,6 @@ QWaylandTabletSeatV2::QWaylandTabletSeatV2(QWaylandTabletManagerV2 *manager, QWa
 
 QWaylandTabletSeatV2::~QWaylandTabletSeatV2()
 {
-    for (auto *tablet : m_tablets)
-        tablet->destroy();
-    for (auto *tool : m_tools)
-        tool->destroy();
-    for (auto *pad : m_pads)
-        pad->destroy();
     qDeleteAll(m_tablets);
     qDeleteAll(m_tools);
     qDeleteAll(m_deadTools);
@@ -254,6 +246,11 @@ QWaylandTabletV2::QWaylandTabletV2(::zwp_tablet_v2 *tablet, const QString &seatN
     d->seatName = seatName;
 }
 
+QWaylandTabletV2::~QWaylandTabletV2()
+{
+    destroy();
+}
+
 void QWaylandTabletV2::zwp_tablet_v2_name(const QString &name)
 {
     QPointingDevicePrivate *d = QPointingDevicePrivate::get(this);
@@ -292,7 +289,6 @@ void QWaylandTabletSeatV2::toolRemoved(QWaylandTabletToolV2 *tool)
 
 void QWaylandTabletV2::zwp_tablet_v2_removed()
 {
-    destroy();
     deleteLater();
 }
 
@@ -316,7 +312,10 @@ QWaylandTabletToolV2::QWaylandTabletToolV2(QWaylandTabletSeatV2 *tabletSeat, ::z
 #endif
 }
 
-QWaylandTabletToolV2::~QWaylandTabletToolV2() = default;
+QWaylandTabletToolV2::~QWaylandTabletToolV2()
+{
+    destroy();
+}
 
 void QWaylandTabletToolV2::zwp_tablet_tool_v2_type(uint32_t tool_type)
 {
@@ -410,7 +409,6 @@ void QWaylandTabletToolV2::zwp_tablet_tool_v2_done()
 
 void QWaylandTabletToolV2::zwp_tablet_tool_v2_removed()
 {
-    destroy();
     m_tabletSeat->toolRemoved(this);
 }
 
@@ -602,6 +600,11 @@ QWaylandTabletPadV2::QWaylandTabletPadV2(::zwp_tablet_pad_v2 *pad)
 {
 }
 
+QWaylandTabletPadV2::~QWaylandTabletPadV2()
+{
+    destroy();
+}
+
 void QWaylandTabletPadV2::zwp_tablet_pad_v2_path(const QString &path)
 {
     QPointingDevicePrivate *d = QPointingDevicePrivate::get(this);
@@ -621,7 +624,6 @@ void QWaylandTabletPadV2::zwp_tablet_pad_v2_done()
 
 void QWaylandTabletPadV2::zwp_tablet_pad_v2_removed()
 {
-    destroy();
     delete this;
 }
 
