@@ -22,6 +22,7 @@
 #include <QtCore/QObject>
 #include <QtCore/QPointer>
 #include <QtCore/QPointF>
+#include <QtCore/QTimer>
 #include <QtGui/QPointingDevice>
 #include <QtGui/QInputDevice>
 
@@ -38,11 +39,18 @@ class QWaylandTabletV2;
 class QWaylandTabletToolV2;
 class QWaylandTabletPadV2;
 
+#if QT_CONFIG(cursor)
+class QWaylandCursorTheme;
+class QWaylandCursorShape;
+template <typename T>
+class CursorSurface;
+#endif
+
 class Q_WAYLANDCLIENT_EXPORT QWaylandTabletManagerV2 : public QtWayland::zwp_tablet_manager_v2
 {
 public:
     explicit QWaylandTabletManagerV2(QWaylandDisplay *display, uint id, uint version);
-    QWaylandTabletSeatV2 *createTabletSeat(QWaylandInputDevice *seat);
+    ~QWaylandTabletManagerV2() override;
 };
 
 class Q_WAYLANDCLIENT_EXPORT QWaylandTabletSeatV2 : public QObject, public QtWayland::zwp_tablet_seat_v2
@@ -52,41 +60,53 @@ public:
     explicit QWaylandTabletSeatV2(QWaylandTabletManagerV2 *manager, QWaylandInputDevice *seat);
     ~QWaylandTabletSeatV2() override;
 
+    QWaylandInputDevice *seat() const { return m_seat; }
+
+    void updateCursor();
+    void toolRemoved(QWaylandTabletToolV2 *tool);
+
 protected:
     void zwp_tablet_seat_v2_tablet_added(struct ::zwp_tablet_v2 *id) override;
     void zwp_tablet_seat_v2_tool_added(struct ::zwp_tablet_tool_v2 *id) override;
     void zwp_tablet_seat_v2_pad_added(struct ::zwp_tablet_pad_v2 *id) override;
 
 private:
+    QWaylandInputDevice *m_seat;
     QList<QWaylandTabletV2 *> m_tablets;
     QList<QWaylandTabletToolV2 *> m_tools;
+    QList<QWaylandTabletToolV2 *> m_deadTools;
     QList<QWaylandTabletPadV2 *> m_pads;
 };
 
-class Q_WAYLANDCLIENT_EXPORT QWaylandTabletV2 : public QObject, public QtWayland::zwp_tablet_v2
+class Q_WAYLANDCLIENT_EXPORT QWaylandTabletV2 : public QPointingDevice, public QtWayland::zwp_tablet_v2
 {
     Q_OBJECT
 public:
-    explicit QWaylandTabletV2(::zwp_tablet_v2 *tablet);
+    explicit QWaylandTabletV2(::zwp_tablet_v2 *tablet, const QString &seatName);
+    ~QWaylandTabletV2();
 
 protected:
-//    void zwp_tablet_v2_name(const QString &name) override;
-//    void zwp_tablet_v2_id(uint32_t vid, uint32_t pid) override;
-//    void zwp_tablet_v2_path(const QString &path) override;
-//    void zwp_tablet_v2_done() override;
+    // callbacks which act as setters
+    void zwp_tablet_v2_name(const QString &name) override;
+    void zwp_tablet_v2_id(uint32_t vid, uint32_t pid) override;
+    void zwp_tablet_v2_path(const QString &path) override;
+    void zwp_tablet_v2_done() override;
     void zwp_tablet_v2_removed() override;
 };
 
-class Q_WAYLANDCLIENT_EXPORT QWaylandTabletToolV2 : public QObject, public QtWayland::zwp_tablet_tool_v2
+class Q_WAYLANDCLIENT_EXPORT QWaylandTabletToolV2 : public QPointingDevice, public QtWayland::zwp_tablet_tool_v2
 {
     Q_OBJECT
 public:
-    explicit QWaylandTabletToolV2(::zwp_tablet_tool_v2 *tool);
+    QWaylandTabletToolV2(QWaylandTabletSeatV2 *tabletSeat, ::zwp_tablet_tool_v2 *tool);
+    ~QWaylandTabletToolV2();
+
+    void updateCursor();
 
 protected:
     void zwp_tablet_tool_v2_type(uint32_t tool_type) override;
     void zwp_tablet_tool_v2_hardware_serial(uint32_t hardware_serial_hi, uint32_t hardware_serial_lo) override;
-//    void zwp_tablet_tool_v2_hardware_id_wacom(uint32_t hardware_id_hi, uint32_t hardware_id_lo) override;
+    void zwp_tablet_tool_v2_hardware_id_wacom(uint32_t hardware_id_hi, uint32_t hardware_id_lo) override;
     void zwp_tablet_tool_v2_capability(uint32_t capability) override;
     void zwp_tablet_tool_v2_done() override;
     void zwp_tablet_tool_v2_removed() override;
@@ -105,14 +125,37 @@ protected:
     void zwp_tablet_tool_v2_frame(uint32_t time) override;
 
 private:
+#if QT_CONFIG(cursor)
+    int idealCursorScale() const;
+    void updateCursorTheme();
+    void cursorTimerCallback();
+    void cursorFrameCallback();
+    CursorSurface<QWaylandTabletToolV2> *getOrCreateCursorSurface();
+#endif
+
+    QWaylandTabletSeatV2 *m_tabletSeat;
 
     // Static state (sent before done event)
     QPointingDevice::PointerType m_pointerType = QPointingDevice::PointerType::Unknown;
     QInputDevice::DeviceType m_tabletDevice = QInputDevice::DeviceType::Unknown;
-    type m_toolType = type_pen;
+    zwp_tablet_tool_v2::type m_toolType = type_pen;
     bool m_hasRotation = false;
     quint64 m_uid = 0;
 
+    uint32_t mEnterSerial = 0;
+#if QT_CONFIG(cursor)
+    struct
+    {
+        QScopedPointer<QWaylandCursorShape> shape;
+        QWaylandCursorTheme *theme = nullptr;
+        int themeBufferScale = 0;
+        QScopedPointer<CursorSurface<QWaylandTabletToolV2>> surface;
+        QTimer frameTimer;
+        bool gotFrameCallback = false;
+        bool gotTimerCallback = false;
+    } mCursor;
+#endif
+
     // Accumulated state (applied on frame event)
     struct State {
         bool down = false;
@@ -129,20 +172,23 @@ private:
         //auto operator<=>(const Point&) const = default; // TODO: use this when upgrading to C++20
         bool operator==(const State &o) const;
     } m_pending, m_applied;
+
+    template <typename T>
+    friend class CursorSurface;
 };
 
-// We don't actually use this, but need to handle the "removed" event to comply with the protocol
-class Q_WAYLANDCLIENT_EXPORT QWaylandTabletPadV2 : public QObject, public QtWayland::zwp_tablet_pad_v2
+class Q_WAYLANDCLIENT_EXPORT QWaylandTabletPadV2 : public QPointingDevice, public QtWayland::zwp_tablet_pad_v2
 {
     Q_OBJECT
 public:
     explicit QWaylandTabletPadV2(::zwp_tablet_pad_v2 *pad);
+    ~QWaylandTabletPadV2();
 
 protected:
 //    void zwp_tablet_pad_v2_group(struct ::zwp_tablet_pad_group_v2 *pad_group) override;
-//    void zwp_tablet_pad_v2_path(const QString &path) override;
-//    void zwp_tablet_pad_v2_buttons(uint32_t buttons) override;
-//    void zwp_tablet_pad_v2_done() override;
+    void zwp_tablet_pad_v2_path(const QString &path) override;
+    void zwp_tablet_pad_v2_buttons(uint32_t buttons) override;
+    void zwp_tablet_pad_v2_done() override;
 //    void zwp_tablet_pad_v2_button(uint32_t time, uint32_t button, uint32_t state) override;
 //    void zwp_tablet_pad_v2_enter(uint32_t serial, struct ::zwp_tablet_v2 *tablet, struct ::wl_surface *surface) override;
 //    void zwp_tablet_pad_v2_leave(uint32_t serial, struct ::wl_surface *surface) override;
