@@ -4,12 +4,14 @@
 
 #include "qwaylandtextinputv1_p.h"
 
+#include "qwaylandinputcontext_p.h"
 #include "qwaylandwindow_p.h"
 #include "qwaylandinputmethodeventbuilder_p.h"
 
 #include <QtCore/qloggingcategory.h>
 #include <QtGui/QGuiApplication>
 #include <QtGui/private/qguiapplication_p.h>
+#include <QtGui/private/qhighdpiscaling_p.h>
 #include <QtGui/qpa/qplatformintegration.h>
 #include <QtGui/qevent.h>
 #include <QtGui/qwindow.h>
@@ -20,8 +22,6 @@
 
 QT_BEGIN_NAMESPACE
 
-Q_DECLARE_LOGGING_CATEGORY(qLcQpaInputMethods)
-
 namespace QtWaylandClient {
 
 namespace {
@@ -45,6 +45,7 @@ QWaylandTextInputv1::~QWaylandTextInputv1()
 {
     if (m_resetCallback)
         wl_callback_destroy(m_resetCallback);
+    zwp_text_input_v1_destroy(object());
 }
 
 void QWaylandTextInputv1::reset()
@@ -127,8 +128,9 @@ void QWaylandTextInputv1::updateState(Qt::InputMethodQueries queries, uint32_t f
     if (queries & Qt::ImCursorRectangle) {
         const QRect &cRect = event.value(Qt::ImCursorRectangle).toRect();
         const QRect &windowRect = QGuiApplication::inputMethod()->inputItemTransform().mapRect(cRect);
-        const QMargins margins = window->frameMargins();
-        const QRect &surfaceRect = windowRect.translated(margins.left(), margins.top());
+        const QRect &nativeRect = QHighDpi::toNativePixels(windowRect, QGuiApplication::focusWindow());
+        const QMargins margins = window->clientSideMargins();
+        const QRect &surfaceRect = nativeRect.translated(margins.left(), margins.top());
         set_cursor_rectangle(surfaceRect.x(), surfaceRect.y(), surfaceRect.width(), surfaceRect.height());
     }
 
