@@ -23,13 +23,13 @@
 
 #include "extensions/qwlqtkey_p.h"
 #include "extensions/qwaylandtextinput.h"
-#if QT_WAYLAND_TEXT_INPUT_V4_WIP
-#include "extensions/qwaylandtextinputv4.h"
-#endif // QT_WAYLAND_TEXT_INPUT_V4_WIP
+#include "extensions/qwaylandtextinputv3.h"
 #include "extensions/qwaylandqttextinputmethod.h"
 
 QT_BEGIN_NAMESPACE
 
+int QWaylandSeatPrivate::max_name = 0;
+
 QWaylandSeatPrivate::QWaylandSeatPrivate(QWaylandSeat *seat) :
 #if QT_CONFIG(wayland_datadevice)
     drag_handle(new QWaylandDrag(seat)),
@@ -88,6 +88,7 @@ void QWaylandSeatPrivate::seat_destroy_resource(wl_seat::Resource *)
 
 void QWaylandSeatPrivate::seat_bind_resource(wl_seat::Resource *resource)
 {
+    wl_seat::send_name(resource->handle, QStringLiteral("seat%1").arg(QString::number(name)));
     // The order of capabilities matches the order defined in the wayland protocol
     wl_seat::send_capabilities(resource->handle, (uint32_t)capabilities);
 }
@@ -115,7 +116,7 @@ void QWaylandSeatPrivate::seat_get_touch(wl_seat::Resource *resource, uint32_t i
 
 /*!
  * \qmltype WaylandSeat
- * \instantiates QWaylandSeat
+ * \nativetype QWaylandSeat
  * \inqmlmodule QtWayland.Compositor
  * \since 5.8
  * \brief Provides access to keyboard, mouse, and touch input.
@@ -153,6 +154,8 @@ QWaylandSeat::QWaylandSeat(QWaylandCompositor *compositor, CapabilityFlags capab
     : QWaylandObject(*new QWaylandSeatPrivate(this))
 {
     Q_D(QWaylandSeat);
+
+    d->name = d->max_name++;
     d->compositor = compositor;
     d->capabilities = capabilityFlags;
     if (compositor->isCreated())
@@ -279,7 +282,7 @@ uint QWaylandSeat::sendTouchPointEvent(QWaylandSurface *surface, int id, const Q
 }
 
 /*!
- * \qmlmethod uint QtWaylandCompositor::WaylandSeat::sendTouchPointPressed(WaylandSurface surface, int id, point position)
+ * \qmlmethod uint QtWayland.Compositor::WaylandSeat::sendTouchPointPressed(WaylandSurface surface, int id, point position)
  *
  * Sends a touch pressed event for the touch point \a id on \a surface with
  * position \a position.
@@ -313,7 +316,7 @@ uint QWaylandSeat::sendTouchPointPressed(QWaylandSurface *surface, int id, const
 }
 
 /*!
- * \qmlmethod void QtWaylandCompositor::WaylandSeat::sendTouchPointReleased(WaylandSurface surface, int id, point position)
+ * \qmlmethod void QtWayland.Compositor::WaylandSeat::sendTouchPointReleased(WaylandSurface surface, int id, point position)
  *
  * Sends a touch released event for the touch point \a id on \a surface with
  * position \a position.
@@ -347,7 +350,7 @@ uint QWaylandSeat::sendTouchPointReleased(QWaylandSurface *surface, int id, cons
 }
 
 /*!
- * \qmlmethod void QtWaylandCompositor::WaylandSeat::sendTouchPointMoved(WaylandSurface surface, int id, point position)
+ * \qmlmethod void QtWayland.Compositor::WaylandSeat::sendTouchPointMoved(WaylandSurface surface, int id, point position)
  *
  * Sends a touch moved event for the touch point \a id on \a surface with
  * position \a position.
@@ -381,7 +384,7 @@ uint QWaylandSeat::sendTouchPointMoved(QWaylandSurface *surface, int id, const Q
 }
 
 /*!
- * \qmlmethod void QtWaylandCompositor::WaylandSeat::sendTouchFrameEvent(WaylandClient client)
+ * \qmlmethod void QtWayland.Compositor::WaylandSeat::sendTouchFrameEvent(WaylandClient client)
  *
  * Sends a frame event to the touch device of a \a client to indicate the end
  * of a series of touch up, down, and motion events.
@@ -399,7 +402,7 @@ void QWaylandSeat::sendTouchFrameEvent(QWaylandClient *client)
 }
 
 /*!
- * \qmlmethod void QtWaylandCompositor::WaylandSeat::sendTouchCancelEvent(WaylandClient client)
+ * \qmlmethod void QtWayland.Compositor::WaylandSeat::sendTouchCancelEvent(WaylandClient client)
  *
  * Sends a cancel event to the touch device of a \a client.
  */
@@ -434,6 +437,11 @@ void QWaylandSeat::sendFullTouchEvent(QWaylandSurface *surface, QTouchEvent *eve
 
 /*!
  * Sends the \a event to the keyboard device.
+ *
+ * \note The \a event should correspond to an actual keyboard key in the current mapping.
+ * For example, \c Qt::Key_Exclam is normally not a separate key: with most keyboards the
+ * exclamation mark is produced with Shift + 1. In that case, to send an exclamation mark
+ * key press event, use \c{QKeyEvent(QEvent::KeyPress, Qt::Key_1, Qt::ShiftModifier)}.
  */
 void QWaylandSeat::sendFullKeyEvent(QKeyEvent *event)
 {
@@ -463,17 +471,15 @@ void QWaylandSeat::sendFullKeyEvent(QKeyEvent *event)
             }
         }
 
-#if QT_WAYLAND_TEXT_INPUT_V4_WIP
-        if (keyboardFocus()->client()->textInputProtocols().testFlag(QWaylandClient::TextInputProtocol::TextInputV4)) {
-            QWaylandTextInputV4 *textInputV4 = QWaylandTextInputV4::findIn(this);
-            if (textInputV4 && !event->text().isEmpty()) {
-                // it will just commit the text for text-input-unstable-v4-wip when keyPress
+        if (keyboardFocus()->client()->textInputProtocols().testFlag(QWaylandClient::TextInputProtocol::TextInputV3)) {
+            QWaylandTextInputV3 *textInputV3 = QWaylandTextInputV3::findIn(this);
+            if (textInputV3 && !event->text().isEmpty()) {
+                // it will just commit the text for text-input-unstable-v3 when keyPress
                 if (event->type() == QEvent::KeyPress)
-                    textInputV4->sendKeyEvent(event);
+                    textInputV3->sendKeyEvent(event);
                 return;
             }
         }
-#endif // QT_WAYLAND_TEXT_INPUT_V4_WIP
     }
 #endif
 
@@ -502,7 +508,7 @@ void QWaylandSeat::sendFullKeyEvent(QKeyEvent *event)
 }
 
 /*!
- * \qmlmethod void QtWaylandCompositor::WaylandSeat::sendKeyEvent(int qtKey, bool pressed)
+ * \qmlmethod void QtWayland.Compositor::WaylandSeat::sendKeyEvent(int qtKey, bool pressed)
  * \since 5.12
  *
  * Sends a key press (if \a pressed is \c true) or release (if \a pressed is \c false)
@@ -513,6 +519,9 @@ void QWaylandSeat::sendFullKeyEvent(QKeyEvent *event)
  * Sends a key press (if \a pressed is \c true) or release (if \a pressed is \c false)
  * event of a key \a qtKey to the keyboard device.
  *
+ * \note This function does not support key events that require modifiers, such as \c Qt::Key_Exclam.
+ * Use \l{sendFullKeyEvent} instead.
+ *
  * \since 5.12
  */
 void QWaylandSeat::sendKeyEvent(int qtKey, bool pressed)
@@ -533,6 +542,104 @@ void QWaylandSeat::sendKeyEvent(int qtKey, bool pressed)
     }
 }
 
+/*!
+ * \qmlmethod void QtWayland.Compositor::WaylandSeat::sendUnicodeKeyPressEvent(uint unicode)
+ * \since 6.7
+ *
+ * Sends a key press event of a UCS4 \a unicode through a text-input protocol.
+ *
+ * \note This function will not work properly if the client does not support the
+ * text-input protocol that the compositor supports.
+ */
+
+/*!
+ * Sends a key press event of a UCS4 \a unicode through a text-input protocol.
+ *
+ * \note This function will not work properly if the client does not support the
+ * text-input protocol that the compositor supports.
+ *
+ * \sa {sendFullKeyEvent} {sendKeyEvent}
+ *
+ * \since 6.7
+ */
+void QWaylandSeat::sendUnicodeKeyPressEvent(uint unicode)
+{
+    sendUnicodeKeyEvent(unicode, QEvent::KeyPress);
+}
+
+/*!
+ * \qmlmethod void QtWayland.Compositor::WaylandSeat::sendUnicodeKeyReleaseEvent(uint unicode)
+ * \since 6.7
+ *
+ * Sends a key release event of a UCS4 \a unicode through a text-input protocol.
+ *
+ * \note This function will not work properly if the client does not support the
+ * text-input protocol that the compositor supports.
+ */
+
+/*!
+ * Sends a key release event of a UCS4 \a unicode through a text-input protocol.
+ *
+ * \note This function will not work properly if the client does not support the
+ * text-input protocol that the compositor supports.
+ *
+ * \sa {sendFullKeyEvent} {sendKeyEvent}
+ *
+ * \since 6.7
+ */
+void QWaylandSeat::sendUnicodeKeyReleaseEvent(uint unicode)
+{
+    sendUnicodeKeyEvent(unicode, QEvent::KeyRelease);
+}
+
+/*!
+ * \internal
+ *
+ * Sends an \a eventType for the UCS4 \a unicode through a text-input protocol.
+ */
+void QWaylandSeat::sendUnicodeKeyEvent(uint unicode, QEvent::Type eventType)
+{
+    if (!keyboardFocus()) {
+        qWarning("Can't send a unicode key event, no keyboard focus, fix the compositor");
+        return;
+    }
+#if QT_CONFIG(im)
+    QString text;
+    text += QChar::fromUcs4(static_cast<char32_t>(unicode));
+
+    QKeyEvent event(eventType, Qt::Key_unknown, Qt::KeyboardModifiers{}, text);
+    if (keyboardFocus()->client()->textInputProtocols().testFlag(QWaylandClient::TextInputProtocol::TextInputV2)) {
+        QWaylandTextInput *textInput = QWaylandTextInput::findIn(this);
+        if (textInput) {
+            textInput->sendKeyEvent(&event);
+            return;
+        }
+    }
+
+    if (keyboardFocus()->client()->textInputProtocols().testFlag(QWaylandClient::TextInputProtocol::QtTextInputMethodV1)) {
+        QWaylandQtTextInputMethod *textInputMethod = QWaylandQtTextInputMethod::findIn(this);
+        if (textInputMethod) {
+            textInputMethod->sendKeyEvent(&event);
+            return;
+        }
+    }
+
+    if (keyboardFocus()->client()->textInputProtocols().testFlag(QWaylandClient::TextInputProtocol::TextInputV3)) {
+        QWaylandTextInputV3 *textInputV3 = QWaylandTextInputV3::findIn(this);
+        if (textInputV3 && !text.isEmpty()) {
+            // it will just commit the text for text-input-unstable-v3 when keyPress
+            if (eventType == QEvent::KeyPress)
+                textInputV3->sendKeyEvent(&event);
+            return;
+        }
+    }
+#else
+    Q_UNUSED(unicode);
+    Q_UNUSED(eventType);
+    qWarning() << "Can't send a unicode key event: Unable to find a text-input protocol.";
+#endif
+}
+
 /*!
  * Returns the keyboard for this input device.
  */
@@ -705,7 +812,7 @@ void QWaylandSeat::handleMouseFocusDestroyed()
 }
 
 
-/*! \qmlsignal void QtWaylandCompositor::QWaylandSeat::keyboardFocusChanged(QWaylandSurface newFocus, QWaylandSurface oldFocus)
+/*! \qmlsignal void QtWayland.Compositor::WaylandSeat::keyboardFocusChanged(QWaylandSurface newFocus, QWaylandSurface oldFocus)
  *
  * This signal is emitted when setKeyboardFocus() is called or when a WaylandQuickItem has focus
  * and the user starts pressing keys.
@@ -725,7 +832,7 @@ void QWaylandSeat::handleMouseFocusDestroyed()
  * \a oldFocus has the surface that lost keyboard focus; or \c nullptr if no surface had focus.
  */
 
-/*! \qmlsignal void QtWaylandCompositor::QWaylandSeat::cursorSurfaceRequest(QWaylandSurface surface, int hotspotX, int hotspotY)
+/*! \qmlsignal void QtWayland.Compositor::WaylandSeat::cursorSurfaceRequest(QWaylandSurface surface, int hotspotX, int hotspotY)
  *
  * This signal is emitted when the client has requested for a specific \a surface to be the mouse
  * cursor. For example, when the user hovers over a particular surface, and you want the cursor
