@@ -195,6 +195,9 @@ QWaylandGLContext::QWaylandGLContext(EGLDisplay eglDisplay, QWaylandDisplay *dis
                                      const QSurfaceFormat &fmt, QPlatformOpenGLContext *share)
     : QEGLPlatformContext(fmt, share, eglDisplay), m_display(display)
 {
+    m_reconnectionWatcher = QObject::connect(m_display, &QWaylandDisplay::connected,
+                                             m_display, [this] { invalidateContext(); });
+
     switch (format().renderableType()) {
     case QSurfaceFormat::OpenVG:
         m_api = EGL_OPENVG_API;
@@ -230,7 +233,7 @@ QWaylandGLContext::QWaylandGLContext(EGLDisplay eglDisplay, QWaylandDisplay *dis
             m_supportNonBlockingSwap = supportNonBlockingSwap != 0;
     }
     if (!m_supportNonBlockingSwap) {
-        qWarning(lcQpaWayland) << "Non-blocking swap buffers not supported."
+        qCWarning(lcQpaWayland) << "Non-blocking swap buffers not supported."
                                << "Subsurface rendering can be affected."
                                << "It may also cause the event loop to freeze in some situations";
     }
@@ -258,8 +261,34 @@ void QWaylandGLContext::destroyTemporaryOffscreenSurface(EGLSurface eglSurface)
     m_wlSurface = nullptr;
 }
 
+void QWaylandGLContext::runGLChecks()
+{
+    bool ok;
+    const int doneCurrentWorkAround = qEnvironmentVariableIntValue("QT_WAYLAND_ENABLE_DONECURRENT_WORKAROUND", &ok);
+    if (ok) {
+        m_doneCurrentWorkAround = doneCurrentWorkAround != 0;
+        if (m_doneCurrentWorkAround)
+            qCDebug(lcQpaWayland) << "Enabling doneCurrent() workaround on request.";
+        else
+            qCDebug(lcQpaWayland) << "Disabling doneCurrent() workaround on request.";
+
+    } else {
+        // Note that even though there is an EGL context current here,
+        // QOpenGLContext and QOpenGLFunctions are not yet usable at this stage.
+        const char *renderer = reinterpret_cast<const char *>(glGetString(GL_RENDERER));
+        if (renderer && strstr(renderer, "Mali")) {
+            qCDebug(lcQpaWayland) << "Enabling doneCurrent() workaround for Mali GPU."
+                                  << "Set QT_WAYLAND_ENABLE_DONECURRENT_WORKAROUND=0 to disable.";
+            m_doneCurrentWorkAround = true;
+        }
+    }
+
+    QEGLPlatformContext::runGLChecks();
+}
+
 QWaylandGLContext::~QWaylandGLContext()
 {
+    QObject::disconnect(m_reconnectionWatcher);
     delete m_blitter;
     m_blitter = nullptr;
     if (m_decorationsContext != EGL_NO_CONTEXT)
@@ -269,17 +298,28 @@ QWaylandGLContext::~QWaylandGLContext()
 void QWaylandGLContext::beginFrame()
 {
     Q_ASSERT(m_currentWindow != nullptr);
-    m_currentWindow->beginFrame();
+    if (m_supportNonBlockingSwap)
+        m_currentWindow->beginFrame();
 }
 
 void QWaylandGLContext::endFrame()
 {
     Q_ASSERT(m_currentWindow != nullptr);
-    m_currentWindow->endFrame();
+    if (m_doneCurrentWorkAround) {
+        doneCurrent();
+        QOpenGLContextPrivate::setCurrentContext(nullptr);
+    }
+
+    if (m_supportNonBlockingSwap)
+        m_currentWindow->endFrame();
 }
 
 bool QWaylandGLContext::makeCurrent(QPlatformSurface *surface)
 {
+    if (!isValid()) {
+        return false;
+    }
+
     // in QWaylandGLContext() we called eglBindAPI with the correct value. However,
     // eglBindAPI's documentation says:
     // "eglBindAPI defines the current rendering API for EGL in the thread it is called from"
@@ -290,6 +330,8 @@ bool QWaylandGLContext::makeCurrent(QPlatformSurface *surface)
     }
 
     m_currentWindow = static_cast<QWaylandEglWindow *>(surface);
+
+    QMutexLocker lock(m_currentWindow->eglSurfaceLock());
     EGLSurface eglSurface = m_currentWindow->eglSurface();
 
     if (!m_currentWindow->needToUpdateContentFBO() && (eglSurface != EGL_NO_SURFACE)) {
@@ -302,8 +344,6 @@ bool QWaylandGLContext::makeCurrent(QPlatformSurface *surface)
 
     if (m_currentWindow->isExposed())
         m_currentWindow->setCanResize(false);
-    if (m_decorationsContext != EGL_NO_CONTEXT && !m_currentWindow->decoration())
-        m_currentWindow->createDecoration();
 
     if (eglSurface == EGL_NO_SURFACE) {
         m_currentWindow->updateSurface(true);
@@ -363,7 +403,8 @@ void QWaylandGLContext::swapBuffers(QPlatformSurface *surface)
         window->waitForFrameSync(100);
     }
     window->handleUpdate();
-    eglSwapBuffers(eglDisplay(), eglSurface);
+    if (!eglSwapBuffers(eglDisplay(), eglSurface))
+        qCWarning(lcQpaWayland, "eglSwapBuffers failed with %#x, surface: %p", eglGetError(), eglSurface);
 
     window->setCanResize(true);
 }
