@@ -1,5 +1,5 @@
 // Copyright (C) 2018 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include "coreprotocol.h"
 #include "datadevice.h"
@@ -77,8 +77,10 @@ void Surface::surface_attach(Resource *resource, wl_resource *buffer, int32_t x,
             m_image = QImage();
     } else {
         QPoint offset(x, y);
+        if (resource->version() < 5)
+            m_pending.commitSpecific.attachOffset = offset;
+
         m_pending.buffer = fromResource<Buffer>(buffer);
-        m_pending.commitSpecific.attachOffset = offset;
         m_pending.commitSpecific.attached = true;
 
         emit attach(buffer, offset);
@@ -109,7 +111,7 @@ void Surface::surface_commit(Resource *resource)
             }
         }
 
-        for (wl_resource *frameCallback : qExchange(m_frameCallbackList, {})) {
+        for (wl_resource *frameCallback : std::exchange(m_frameCallbackList, {})) {
             auto time = m_wlCompositor->m_compositor->currentTimeMilliseconds();
             wl_callback_send_done(frameCallback, time);
             wl_resource_destroy(frameCallback);
@@ -143,6 +145,13 @@ void Surface::surface_frame(Resource *resource, uint32_t callback)
     }
 }
 
+void Surface::surface_offset(Resource *resource, int32_t x, int32_t y)
+{
+    Q_UNUSED(resource);
+    QPoint offset(x, y);
+    m_pending.commitSpecific.attachOffset = offset;
+}
+
 bool WlCompositor::isClean() {
     for (auto *surface : std::as_const(m_surfaces)) {
         if (!CursorRole::fromSurface(surface)) {
@@ -226,11 +235,14 @@ void Output::output_bind_resource(QtWaylandServer::wl_output::Resource *resource
 
     if (m_version >= WL_OUTPUT_DONE_SINCE_VERSION)
         wl_output::send_done(resource->handle);
+
+    Q_EMIT outputBound(resource);
 }
 
 // Seat stuff
-Seat::Seat(CoreCompositor *compositor, uint capabilities, int version) //TODO: check version
+Seat::Seat(CoreCompositor *compositor, uint capabilities, int version, const QString &seatName) //TODO: check version
     : QtWaylandServer::wl_seat(compositor->m_display, version)
+    , m_seatName(seatName)
     , m_compositor(compositor)
 {
     setCapabilities(capabilities);
@@ -336,7 +348,7 @@ uint Pointer::sendEnter(Surface *surface, const QPointF &position)
 
     uint serial = m_seat->m_compositor->nextSerial();
     m_enterSerials << serial;
-    m_cursorRole = nullptr; // According to the protocol, the pointer image is undefined after enter
+    m_cursorRole.clear(); // According to the protocol, the pointer image is undefined after enter
 
     wl_client *client = surface->resource()->client();
     const auto pointerResources = resourceMap().values(client);
@@ -423,18 +435,36 @@ void Pointer::sendFrame(wl_client *client)
         send_frame(r->handle);
 }
 
+void Pointer::sendAxisValue120(wl_client *client, QtWaylandServer::wl_pointer::axis axis, int value120)
+{
+    const auto pointerResources = resourceMap().values(client);
+    for (auto *r : pointerResources)
+        send_axis_value120(r->handle, axis, value120);
+}
+
+void Pointer::sendAxisRelativeDirection(wl_client *client, QtWaylandServer::wl_pointer::axis axis, QtWaylandServer::wl_pointer::axis_relative_direction direction)
+{
+    const auto pointerResources = resourceMap().values(client);
+    for (auto *r : pointerResources)
+        send_axis_relative_direction(r->handle, axis, direction);
+}
+
 void Pointer::pointer_set_cursor(Resource *resource, uint32_t serial, wl_resource *surface, int32_t hotspot_x, int32_t hotspot_y)
 {
     Q_UNUSED(resource);
-    auto *s = fromResource<Surface>(surface);
-    QVERIFY(s);
 
-    if (s->m_role) {
-        m_cursorRole = CursorRole::fromSurface(s);
-        QVERIFY(m_cursorRole);
+    if (!surface) {
+        m_cursorRole = nullptr;
     } else {
-        m_cursorRole = new CursorRole(s); //TODO: make sure we don't leak CursorRole
-        s->m_role = m_cursorRole;
+        auto *s = fromResource<Surface>(surface);
+        QVERIFY(s);
+        if (s->m_role) {
+            m_cursorRole = CursorRole::fromSurface(s);
+            QVERIFY(m_cursorRole);
+        } else {
+            m_cursorRole = new CursorRole(s); //TODO: make sure we don't leak CursorRole
+            s->m_role = m_cursorRole;
+        }
     }
 
     // Directly checking the last serial would be racy, we may just have sent leaves/enters which
@@ -493,6 +523,13 @@ void Touch::sendFrame(wl_client *client)
         send_frame(r->handle);
 }
 
+void Touch::sendCancel(wl_client *client)
+{
+    const auto touchResources = resourceMap().values(client);
+    for (auto *r : touchResources)
+        send_cancel(r->handle);
+}
+
 uint Keyboard::sendEnter(Surface *surface)
 {
     auto serial = m_seat->m_compositor->nextSerial();
