@@ -1,8 +1,8 @@
 // Copyright (C) 2018 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// Copyright (C) 2023 David Edmundson <davidedmundson@kde.org>
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include "mockcompositor.h"
-#include <QtOpenGL/QOpenGLWindow>
 #include <QtGui/QRasterWindow>
 #include <QtGui/qpa/qplatformnativeinterface.h>
 #include <QtWaylandClient/private/wayland-wayland-client-protocol.h>
@@ -14,23 +14,34 @@ class tst_xdgshell : public QObject, private DefaultCompositor
 {
     Q_OBJECT
 private slots:
+    void init();
     void cleanup() { QTRY_VERIFY2(isClean(), qPrintable(dirtyMessage())); }
     void showMinimized();
     void basicConfigure();
     void configureSize();
     void configureStates();
+    void configureBounds();
     void popup();
     void tooltipOnPopup();
     void tooltipAndSiblingPopup();
     void switchPopups();
     void hidePopupParent();
     void pongs();
+    void minMaxSize_data();
     void minMaxSize();
     void windowGeometry();
     void foreignSurface();
     void nativeResources();
+    void suspended();
+    void initiallySuspended();
+    void modality();
 };
 
+void tst_xdgshell::init()
+{
+    setenv("QT_WAYLAND_DISABLE_WINDOWDECORATION", "1", 1);
+}
+
 void tst_xdgshell::showMinimized()
 {
     // On xdg-shell there's really no way for the compositor to tell the window if it's minimized
@@ -54,13 +65,13 @@ void tst_xdgshell::basicConfigure()
     window.show();
     QCOMPOSITOR_TRY_VERIFY(xdgToplevel());
 
-    QSignalSpy configureSpy(exec([=] { return xdgSurface(); }), &XdgSurface::configureCommitted);
+    QSignalSpy configureSpy(exec([&] { return xdgSurface(); }), &XdgSurface::configureCommitted);
 
     QTRY_VERIFY(window.isVisible());
     // The window should not be exposed before the first xdg_surface configure event
     QTRY_VERIFY(!window.isExposed());
 
-    exec([=] {
+    exec([&] {
         xdgToplevel()->sendConfigure({0, 0}, {}); // Let the window decide the size
     });
 
@@ -68,9 +79,9 @@ void tst_xdgshell::basicConfigure()
     QTRY_VERIFY(!window.isExposed()); //Window should not be exposed before the first configure event
     QVERIFY(configureSpy.isEmpty());
 
-    const uint serial = exec([=] { return nextSerial(); });
+    const uint serial = exec([&] { return nextSerial(); });
 
-    exec([=] {
+    exec([&] {
         xdgSurface()->sendConfigure(serial);
     });
 
@@ -96,17 +107,17 @@ void tst_xdgshell::configureSize()
     window.show();
     QCOMPOSITOR_TRY_VERIFY(xdgToplevel());
 
-    QSignalSpy configureSpy(exec([=] { return xdgSurface(); }), &XdgSurface::configureCommitted);
+    QSignalSpy configureSpy(exec([&] { return xdgSurface(); }), &XdgSurface::configureCommitted);
 
     const QSize configureSize(60, 40);
 
-    exec([=] {
+    exec([&] {
         xdgToplevel()->sendCompleteConfigure(configureSize);
     });
 
     QTRY_COMPARE(configureSpy.size(), 1);
 
-    exec([=] {
+    exec([&] {
         Buffer *buffer = xdgToplevel()->surface()->m_committed.buffer;
         QVERIFY(buffer);
         QCOMPARE(buffer->size(), configureSize);
@@ -122,7 +133,7 @@ void tst_xdgshell::configureStates()
     QCOMPOSITOR_TRY_VERIFY(xdgToplevel());
 
     const QSize windowedSize(320, 240);
-    const uint windowedSerial = exec([=] {
+    const uint windowedSerial = exec([&] {
         return xdgToplevel()->sendCompleteConfigure(windowedSize, { XdgToplevel::state_activated });
     });
     QCOMPOSITOR_TRY_COMPARE(xdgSurface()->m_committedConfigureSerial, windowedSerial);
@@ -140,7 +151,7 @@ void tst_xdgshell::configureStates()
             Qt::WindowActive)); // Just make sure it eventually get's set correctly
 
     const QSize screenSize(640, 480);
-    const uint maximizedSerial = exec([=] {
+    const uint maximizedSerial = exec([&] {
         return xdgToplevel()->sendCompleteConfigure(screenSize, { XdgToplevel::state_activated, XdgToplevel::state_maximized });
     });
     QCOMPOSITOR_TRY_COMPARE(xdgSurface()->m_committedConfigureSerial, maximizedSerial);
@@ -149,7 +160,7 @@ void tst_xdgshell::configureStates()
     QCOMPARE(window.frameGeometry().size(), screenSize);
 //    QCOMPARE(window.frameGeometry().topLeft(), QPoint()); // TODO: this doesn't currently work when window decorations are enabled
 
-    const uint fullscreenSerial = exec([=] {
+    const uint fullscreenSerial = exec([&] {
         return xdgToplevel()->sendCompleteConfigure(screenSize, { XdgToplevel::state_activated, XdgToplevel::state_fullscreen });
     });
     QCOMPOSITOR_TRY_COMPARE(xdgSurface()->m_committedConfigureSerial, fullscreenSerial);
@@ -159,7 +170,7 @@ void tst_xdgshell::configureStates()
 //    QCOMPARE(window.frameGeometry().topLeft(), QPoint()); // TODO: this doesn't currently work when window decorations are enabled
 
     // The window should remember its original size
-    const uint restoreSerial = exec([=] {
+    const uint restoreSerial = exec([&] {
         return xdgToplevel()->sendCompleteConfigure({0, 0}, { XdgToplevel::state_activated });
     });
     QCOMPOSITOR_TRY_COMPARE(xdgSurface()->m_committedConfigureSerial, restoreSerial);
@@ -170,6 +181,30 @@ void tst_xdgshell::configureStates()
     QVERIFY(qunsetenv("QT_WAYLAND_FRAME_CALLBACK_TIMEOUT"));
 }
 
+void tst_xdgshell::configureBounds()
+{
+    QRasterWindow window;
+    window.resize(1280, 1024);
+    window.show();
+    QCOMPOSITOR_TRY_VERIFY(xdgToplevel());
+
+    // Take xdg_toplevel.configure_bounds into account only if the configure event has 0x0 size.
+    const uint serial1 = exec([&] {
+        xdgToplevel()->sendConfigureBounds(QSize(800, 600));
+        return xdgToplevel()->sendCompleteConfigure(QSize(0, 0), { XdgToplevel::state_activated });
+    });
+    QCOMPOSITOR_TRY_COMPARE(xdgSurface()->m_committedConfigureSerial, serial1);
+    QCOMPARE(window.frameGeometry().size(), QSize(800, 600));
+
+    // Window size in xdg_toplevel configure events takes precedence over the configure bounds.
+    const uint serial2 = exec([&] {
+        xdgToplevel()->sendConfigureBounds(QSize(800, 600));
+        return xdgToplevel()->sendCompleteConfigure(QSize(1600, 900), { XdgToplevel::state_activated });
+    });
+    QCOMPOSITOR_TRY_COMPARE(xdgSurface()->m_committedConfigureSerial, serial2);
+    QCOMPARE(window.frameGeometry().size(), QSize(1600, 900));
+}
+
 void tst_xdgshell::popup()
 {
     class Window : public QRasterWindow {
@@ -190,11 +225,11 @@ void tst_xdgshell::popup()
     window.show();
 
     QCOMPOSITOR_TRY_VERIFY(xdgToplevel());
-    QSignalSpy toplevelConfigureSpy(exec([=] { return xdgSurface(); }), &XdgSurface::configureCommitted);
-    exec([=] { xdgToplevel()->sendCompleteConfigure(); });
+    QSignalSpy toplevelConfigureSpy(exec([&] { return xdgSurface(); }), &XdgSurface::configureCommitted);
+    exec([&] { xdgToplevel()->sendCompleteConfigure(); });
     QTRY_COMPARE(toplevelConfigureSpy.size(), 1);
 
-    uint clickSerial = exec([=] {
+    uint clickSerial = exec([&] {
         auto *surface = xdgToplevel()->surface();
         auto *p = pointer();
         auto *c = client();
@@ -208,21 +243,21 @@ void tst_xdgshell::popup()
 
     QTRY_VERIFY(window.m_popup);
     QCOMPOSITOR_TRY_VERIFY(xdgPopup());
-    QSignalSpy popupConfigureSpy(exec([=] { return xdgPopup()->m_xdgSurface; }), &XdgSurface::configureCommitted);
+    QSignalSpy popupConfigureSpy(exec([&] { return xdgPopup()->m_xdgSurface; }), &XdgSurface::configureCommitted);
     QCOMPOSITOR_TRY_VERIFY(xdgPopup()->m_grabbed);
     QCOMPOSITOR_TRY_COMPARE(xdgPopup()->m_grabSerial, clickSerial);
 
     QRasterWindow *popup = window.m_popup.get();
     QVERIFY(!popup->isExposed()); // wait for configure
 
-    //TODO: Verify it works with a different configure window geometry
-    exec([=] { xdgPopup()->sendConfigure(QRect(100, 100, 100, 100)); });
+    QRect rect1 = QRect(100, 100, 100, 100);
+    exec([&] { xdgPopup()->sendConfigure(rect1); });
 
     // Nothing should happen before the *xdg_surface* configure
     QTRY_VERIFY(!popup->isExposed()); // Popup shouldn't be exposed before the first configure event
     QVERIFY(popupConfigureSpy.isEmpty());
 
-    const uint configureSerial = exec([=] {
+    const uint configureSerial = exec([&] {
         return xdgPopup()->m_xdgSurface->sendConfigure();
     });
 
@@ -232,6 +267,18 @@ void tst_xdgshell::popup()
     // The client is now going to ack the configure
     QTRY_COMPARE(popupConfigureSpy.size(), 1);
     QCOMPARE(popupConfigureSpy.takeFirst().at(0).toUInt(), configureSerial);
+    QCOMPARE(popup->geometry(), rect1);
+
+    QRect rect2 = QRect(50, 50, 150, 150);
+    exec([&] { xdgPopup()->sendConfigure(rect2); });
+
+    const uint configureSerial2 = exec([&] {
+        return xdgPopup()->m_xdgSurface->sendConfigure();
+    });
+
+    QTRY_COMPARE(popupConfigureSpy.size(), 1);
+    QCOMPARE(popupConfigureSpy.takeFirst().at(0).toUInt(), configureSerial2);
+    QCOMPARE(popup->geometry(), rect2);
 
     // And attach a buffer
     exec([&] {
@@ -276,10 +323,10 @@ void tst_xdgshell::tooltipOnPopup()
     window.show();
 
     QCOMPOSITOR_TRY_VERIFY(xdgToplevel());
-    exec([=] { xdgToplevel()->sendCompleteConfigure(); });
+    exec([&] { xdgToplevel()->sendCompleteConfigure(); });
     QCOMPOSITOR_TRY_VERIFY(xdgToplevel()->m_xdgSurface->m_committedConfigureSerial);
 
-    exec([=] {
+    exec([&] {
         auto *surface = xdgToplevel()->surface();
         auto *p = pointer();
         auto *c = client();
@@ -293,11 +340,11 @@ void tst_xdgshell::tooltipOnPopup()
     });
 
     QCOMPOSITOR_TRY_VERIFY(xdgPopup());
-    exec([=] { xdgPopup()->sendCompleteConfigure(QRect(100, 100, 100, 100)); });
+    exec([&] { xdgPopup()->sendCompleteConfigure(QRect(100, 100, 100, 100)); });
     QCOMPOSITOR_TRY_VERIFY(xdgPopup()->m_xdgSurface->m_committedConfigureSerial);
     QCOMPOSITOR_TRY_VERIFY(xdgPopup()->m_grabbed);
 
-    exec([=] {
+    exec([&] {
         auto *surface = xdgPopup()->surface();
         auto *p = pointer();
         auto *c = client();
@@ -309,7 +356,7 @@ void tst_xdgshell::tooltipOnPopup()
     });
 
     QCOMPOSITOR_TRY_VERIFY(xdgPopup(1));
-    exec([=] { xdgPopup(1)->sendCompleteConfigure(QRect(100, 100, 100, 100)); });
+    exec([&] { xdgPopup(1)->sendCompleteConfigure(QRect(100, 100, 100, 100)); });
     QCOMPOSITOR_TRY_VERIFY(xdgPopup(1)->m_xdgSurface->m_committedConfigureSerial);
     QCOMPOSITOR_TRY_VERIFY(!xdgPopup(1)->m_grabbed);
 
@@ -358,10 +405,10 @@ void tst_xdgshell::tooltipAndSiblingPopup()
     window.show();
 
     QCOMPOSITOR_TRY_VERIFY(xdgToplevel());
-    exec([=] { xdgToplevel()->sendCompleteConfigure(); });
+    exec([&] { xdgToplevel()->sendCompleteConfigure(); });
     QCOMPOSITOR_TRY_VERIFY(xdgToplevel()->m_xdgSurface->m_committedConfigureSerial);
 
-    exec([=] {
+    exec([&] {
         auto *surface = xdgToplevel()->surface();
         auto *p = pointer();
         auto *c = client();
@@ -375,11 +422,11 @@ void tst_xdgshell::tooltipAndSiblingPopup()
     });
 
     QCOMPOSITOR_TRY_VERIFY(xdgPopup());
-    exec([=] { xdgPopup()->sendCompleteConfigure(QRect(100, 100, 100, 100)); });
+    exec([&] { xdgPopup()->sendCompleteConfigure(QRect(100, 100, 100, 100)); });
     QCOMPOSITOR_TRY_VERIFY(xdgPopup()->m_xdgSurface->m_committedConfigureSerial);
     QCOMPOSITOR_TRY_VERIFY(!xdgPopup()->m_grabbed);
 
-    exec([=] {
+    exec([&] {
         auto *surface = xdgPopup()->surface();
         auto *p = pointer();
         auto *c = client();
@@ -391,7 +438,7 @@ void tst_xdgshell::tooltipAndSiblingPopup()
     });
 
     QCOMPOSITOR_TRY_VERIFY(xdgPopup(1));
-    exec([=] { xdgPopup(1)->sendCompleteConfigure(QRect(100, 100, 100, 100)); });
+    exec([&] { xdgPopup(1)->sendCompleteConfigure(QRect(100, 100, 100, 100)); });
     QCOMPOSITOR_TRY_VERIFY(xdgPopup(1)->m_xdgSurface->m_committedConfigureSerial);
     QCOMPOSITOR_TRY_VERIFY(xdgPopup(1)->m_grabbed);
 
@@ -446,10 +493,10 @@ void tst_xdgshell::switchPopups()
     window.show();
 
     QCOMPOSITOR_TRY_VERIFY(xdgToplevel());
-    exec([=] { xdgToplevel()->sendCompleteConfigure(); });
+    exec([&] { xdgToplevel()->sendCompleteConfigure(); });
     QCOMPOSITOR_TRY_VERIFY(xdgToplevel()->m_xdgSurface->m_committedConfigureSerial);
 
-    exec([=] {
+    exec([&] {
         auto *surface = xdgToplevel()->surface();
         auto *p = pointer();
         auto *c = client();
@@ -463,13 +510,13 @@ void tst_xdgshell::switchPopups()
     });
 
     QCOMPOSITOR_TRY_VERIFY(xdgPopup());
-    exec([=] { xdgPopup()->sendCompleteConfigure(QRect(100, 100, 100, 100)); });
+    exec([&] { xdgPopup()->sendCompleteConfigure(QRect(100, 100, 100, 100)); });
     QCOMPOSITOR_TRY_VERIFY(xdgPopup()->m_xdgSurface->m_committedConfigureSerial);
     QCOMPOSITOR_TRY_VERIFY(xdgPopup()->m_grabbed);
 
-    QSignalSpy firstDestroyed(exec([=] { return xdgPopup(); }), &XdgPopup::destroyRequested);
+    QSignalSpy firstDestroyed(exec([&] { return xdgPopup(); }), &XdgPopup::destroyRequested);
 
-    exec([=] {
+    exec([&] {
         auto *surface = xdgToplevel()->surface();
         auto *p = pointer();
         auto *c = client();
@@ -494,7 +541,7 @@ void tst_xdgshell::switchPopups()
     QCOMPOSITOR_TRY_VERIFY(xdgPopup()->m_parentXdgSurface == xdgToplevel()->m_xdgSurface);
 
     // For good measure just check that configuring works as usual
-    exec([=] { xdgPopup()->sendCompleteConfigure(QRect(100, 100, 100, 100)); });
+    exec([&] { xdgPopup()->sendCompleteConfigure(QRect(100, 100, 100, 100)); });
     QCOMPOSITOR_TRY_VERIFY(xdgPopup()->m_xdgSurface->m_committedConfigureSerial);
 }
 
@@ -518,10 +565,10 @@ void tst_xdgshell::hidePopupParent()
     window.show();
 
     QCOMPOSITOR_TRY_VERIFY(xdgToplevel());
-    exec([=] { xdgToplevel()->sendCompleteConfigure(); });
+    exec([&] { xdgToplevel()->sendCompleteConfigure(); });
     QCOMPOSITOR_TRY_VERIFY(xdgToplevel()->m_xdgSurface->m_committedConfigureSerial);
 
-    exec([=] {
+    exec([&] {
         auto *surface = xdgToplevel()->surface();
         auto *p = pointer();
         auto *c = client();
@@ -532,7 +579,7 @@ void tst_xdgshell::hidePopupParent()
         p->sendFrame(c);
     });
     QCOMPOSITOR_TRY_VERIFY(xdgPopup());
-    exec([=] {
+    exec([&] {
         xdgPopup()->sendConfigure(QRect(100, 100, 100, 100));
         xdgPopup()->m_xdgSurface->sendConfigure();
     });
@@ -553,9 +600,9 @@ void tst_xdgshell::pongs()
     // Verify that the client has bound to the global
     QCOMPOSITOR_TRY_COMPARE(get<XdgWmBase>()->resourceMap().size(), 1);
 
-    QSignalSpy pongSpy(exec([=] { return get<XdgWmBase>(); }), &XdgWmBase::pong);
-    const uint serial = exec([=] { return nextSerial(); });
-    exec([=] {
+    QSignalSpy pongSpy(exec([&] { return get<XdgWmBase>(); }), &XdgWmBase::pong);
+    const uint serial = exec([&] { return nextSerial(); });
+    exec([&] {
         auto *base = get<XdgWmBase>();
         wl_resource *resource = base->resourceMap().first()->handle;
         base->send_ping(resource, serial);
@@ -564,25 +611,69 @@ void tst_xdgshell::pongs()
     QCOMPARE(pongSpy.first().at(0).toUInt(), serial);
 }
 
+void tst_xdgshell::minMaxSize_data()
+{
+    QTest::addColumn<QSize>("initialMinSize");
+    QTest::addColumn<QSize>("initialMaxSize");
+    QTest::addColumn<QSize>("nextMinSize");
+    QTest::addColumn<QSize>("nextMaxSize");
+    QTest::addColumn<QSize>("expectedInitialMinSize");
+    QTest::addColumn<QSize>("expectedInitialMaxSize");
+    QTest::addColumn<QSize>("expectedNextMinSize");
+    QTest::addColumn<QSize>("expectedNextMaxSize");
+
+    QTest::newRow("onlyMinSize") << QSize(50, 60) << QSize() << QSize(500, 600) << QSize()
+                                 << QSize(50, 60) << QSize(0, 0) << QSize(500, 600) << QSize(0, 0);
+
+    QTest::newRow("onlyMaxSize") << QSize() << QSize(70, 80) << QSize() << QSize(700, 800)
+                                 << QSize(0,0 ) << QSize(70, 80) << QSize(0, 0) << QSize(700, 800);
+
+    QTest::newRow("maxIsSentAsZero") << QSize() << QSize(QWINDOWSIZE_MAX, QWINDOWSIZE_MAX) << QSize() << QSize()
+                                 << QSize(0,0 ) << QSize(0, 0) << QSize(0, 0) << QSize(0, 0);
+
+
+    QTest::newRow("fullHints") << QSize(50, 60) << QSize(700, 800) << QSize(500, 600) << QSize(710, 810)
+                               << QSize(50, 60) << QSize(700, 800) << QSize(500, 600) << QSize(710, 810);
+
+    // setting a minimum above the maximum is not allowed, we should no-op
+    QTest::newRow("invalidResize") << QSize(50, 60) << QSize(100, 100) << QSize(500, 600) << QSize(100, 100)
+                                   << QSize(50, 60) << QSize(100, 100) << QSize(50, 60) << QSize(100, 100);}
+
 void tst_xdgshell::minMaxSize()
 {
+    QFETCH(QSize, initialMinSize);
+    QFETCH(QSize, initialMaxSize);
+
+    QFETCH(QSize, expectedInitialMinSize);
+    QFETCH(QSize, expectedInitialMaxSize);
+
     QRasterWindow window;
-    window.setMinimumSize(QSize(100, 100));
-    window.setMaximumSize(QSize(1000, 1000));
-    window.resize(400, 320);
+    if (initialMinSize.isValid())
+        window.setMinimumSize(initialMinSize);
+    if (initialMaxSize.isValid())
+        window.setMaximumSize(initialMaxSize);
     window.show();
     QCOMPOSITOR_TRY_VERIFY(xdgToplevel());
 
+    exec([&] { xdgToplevel()->sendCompleteConfigure(); });
+
     // we don't roundtrip with our configuration the initial commit should be correct
 
-    QCOMPOSITOR_TRY_COMPARE(xdgToplevel()->m_committed.minSize, QSize(100, 100));
-    QCOMPOSITOR_TRY_COMPARE(xdgToplevel()->m_committed.maxSize, QSize(1000, 1000));
+    QCOMPOSITOR_TRY_COMPARE(xdgToplevel()->m_committed.minSize, expectedInitialMinSize);
+    QCOMPOSITOR_TRY_COMPARE(xdgToplevel()->m_committed.maxSize, expectedInitialMaxSize);
 
-    window.setMaximumSize(QSize(500, 400));
-    QCOMPOSITOR_TRY_COMPARE(xdgToplevel()->m_committed.maxSize, QSize(500, 400));
+    QFETCH(QSize, nextMinSize);
+    QFETCH(QSize, expectedNextMinSize);
+    window.setMinimumSize(nextMinSize);
+    window.update();
+    QCOMPOSITOR_TRY_COMPARE(xdgToplevel()->m_committed.minSize, expectedNextMinSize);
 
-    window.setMinimumSize(QSize(50, 40));
-    QCOMPOSITOR_TRY_COMPARE(xdgToplevel()->m_committed.minSize, QSize(50, 40));
+    QFETCH(QSize, nextMaxSize);
+    QFETCH(QSize, expectedNextMaxSize);
+
+    window.setMaximumSize(nextMaxSize);
+    window.update();
+    QCOMPOSITOR_TRY_COMPARE(xdgToplevel()->m_committed.maxSize, expectedNextMaxSize);
 }
 
 void tst_xdgshell::windowGeometry()
@@ -592,7 +683,7 @@ void tst_xdgshell::windowGeometry()
     window.show();
     QCOMPOSITOR_TRY_VERIFY(xdgToplevel());
 
-    exec([=] { xdgToplevel()->sendCompleteConfigure(); });
+    exec([&] { xdgToplevel()->sendCompleteConfigure(); });
 
     QSize marginsSize;
     marginsSize.setWidth(window.frameMargins().left() + window.frameMargins().right());
@@ -624,7 +715,7 @@ void tst_xdgshell::foreignSurface()
 
     // Just do something to make sure we don't destroy the surface before
     // the pointer events above are handled.
-    QSignalSpy spy(exec([=] { return surface(newSurfaceIndex); }), &Surface::commit);
+    QSignalSpy spy(exec([&] { return surface(newSurfaceIndex); }), &Surface::commit);
     wl_surface_commit(foreignSurface);
     QTRY_COMPARE(spy.size(), 1);
 
@@ -649,5 +740,70 @@ void tst_xdgshell::nativeResources()
     QCOMPARE(xdg_popup_proxy, nullptr);
 }
 
+void tst_xdgshell::suspended()
+{
+    QRasterWindow window;
+    window.resize(400, 320);
+    window.show();
+    QVERIFY(!window.isExposed()); // not exposed until we're configured
+    QCOMPOSITOR_TRY_VERIFY(xdgToplevel());
+
+    exec([&] { xdgToplevel()->sendCompleteConfigure(); });
+    QCOMPOSITOR_TRY_VERIFY(xdgToplevel()->m_xdgSurface->m_committedConfigureSerial);
+    QTRY_VERIFY(window.isExposed());
+
+    exec([&] { xdgToplevel()->sendCompleteConfigure(QSize(), {XdgToplevel::state_suspended}); });
+    QTRY_VERIFY(!window.isExposed());
+
+    exec([&] { xdgToplevel()->sendCompleteConfigure(QSize(), {}); });
+    QTRY_VERIFY(window.isExposed());
+}
+
+void tst_xdgshell::initiallySuspended()
+{
+    QRasterWindow window;
+    window.resize(400, 320);
+    window.show();
+    QVERIFY(!window.isExposed());
+    QCOMPOSITOR_TRY_VERIFY(xdgToplevel());
+    exec([&] { xdgToplevel()->sendCompleteConfigure(QSize(), {XdgToplevel::state_suspended}); });
+    QVERIFY(!window.isExposed());
+}
+
+void tst_xdgshell::modality()
+{
+    QRasterWindow parent;
+    parent.resize(400, 320);
+    parent.show();
+
+    QRasterWindow child;
+    child.resize(400, 320);
+    child.setTransientParent(&parent);
+    child.show();
+    QCOMPOSITOR_TRY_VERIFY(xdgToplevel(1));
+    QCOMPOSITOR_VERIFY(!xdgDialog());
+
+    child.hide();
+    child.setModality(Qt::WindowModal);
+    child.show();
+    QCOMPOSITOR_TRY_VERIFY(xdgDialog());
+    QCOMPOSITOR_VERIFY(xdgDialog()->modal);
+
+    child.hide();
+    QCOMPOSITOR_TRY_VERIFY(!xdgDialog());
+
+    child.setModality(Qt::ApplicationModal);
+    child.show();
+    QCOMPOSITOR_TRY_VERIFY(xdgDialog());
+    QCOMPOSITOR_VERIFY(xdgDialog()->modal);
+
+    child.hide();
+    QCOMPOSITOR_TRY_VERIFY(!xdgDialog());
+
+    child.show();
+    child.setModality(Qt::NonModal);
+    QCOMPOSITOR_TRY_VERIFY(!xdgDialog());
+}
+
 QCOMPOSITOR_TEST_MAIN(tst_xdgshell)
 #include "tst_xdgshell.moc"
